from .channel_interface import ChannelInterface
import logging


class ModProbeTemp(ChannelInterface):
    logger = logging.getLogger(__name__)

    def __init__(self, device_file, hardware_serial_no=None, serial_no=None, *args, **kwargs):
        self.device_file = device_file
        self.type = self.__name__
        super().__init__(hardware_serial_no=hardware_serial_no, serial_no=serial_no, *args, **kwargs)

    def get_raw_data(self):
        f = open(self.device_file, 'r')
        lines = f.readlines()
        f.close()
        return lines



from hardware_interfaces import mod_probe, dht22, mcp3008, veml_7700
from initializers.homekit_interfaces.first_time_kit_setup import interface_channel_with_homekit_accessory
import board


def configure_dht22(sensor_hub):
    num_dht22s = int(input("How many DHT22s (humidity/temperature) are connected? ")) or 0
    for i in range(num_dht22s):
        gpio_pin = int(input("Which gpio pin is this connected to? ")) or board.D17
        dht22_interface = dht22.DHT22(gpio_pin)
        sensor_hub.id_hardware_map.append(dht22_interface)


def configure_mcp3008(sensor_hub):
    num_mcp3008s = int(input("How many MCP3008s (analog to digital converter) are connected? "))
    if num_mcp3008s > 0:
        for i in range(num_mcp3008s):
            # sck = board.SCK, miso = board.MISO, mosi = board.MOSI, cs_pin = board.D5
            sck = int(input("Which pin is sck? ")) or board.SCK
            miso = int(input("Which pin is miso? ")) or board.MISO
            mosi = int(input("Which pin is mosi? ")) or board.MOSI
            cs_pin = int(input("Which pin is cs_pin? ")) or board.D5
            mcp3008_interface = mcp3008.MCP3008(sck=sck, miso=miso, mosi=mosi, cs_pin=cs_pin)
            sensor_hub.id_hardware_map.append(mcp3008_interface)

            num_channels = int(input("How many channels of the MCP3008 are you using?"))
            for j in range(num_channels):
                channel = mcp3008_interface.create_analog_channel()
                interface_channel_with_homekit_accessory(channel)


def configure_mod_probe(sensor_hub):
    num_mod_probe = int(input("How many modprobes (soil temp) are connected? ")) or 0
    for i in range(num_mod_probe):
        mod_probe_interface = mod_probe.ModProbe()
        channel = mod_probe_interface.create_channel()
        interface_channel_with_homekit_accessory(channel)
        sensor_hub.id_hardware_map.append(mod_probe_interface)


def configure_pi_picos(sensor_hub):
    global interrupts
    num_pi_picos = int(input("How many pi picos are connected? "))
    for i in range(num_pi_picos):
        interrupt = int(input("Enter one interrupt pin: "))
        interrupts.append(interrupt)


def configure_veml(sensor_hub):
    num_veml = int(input("How many veml (light sensor) are connected? "))
    for i in range(num_veml):
        scl = int(input("Which pin is scl? ")) or board.SCL
        sda = int(input("Which pin is sda? ")) or board.SDA
        veml7700_interface = veml_7700.VEML7700(scl, sda)
        sensor_hub.id_hardware_map.append(veml7700_interface)


def first_run(sensor_hub):
    configure_dht22(sensor_hub)
    configure_veml(sensor_hub)
    configure_pi_picos(sensor_hub)
    configure_mod_probe(sensor_hub)
    configure_mcp3008()


import glob
from .hardware_interface import HardwareInterface
from .channels.mod_probe_temp import ModProbeTemp


# os.system('modprobe w1-gpio')
# os.system('modprobe w1-therm')

class ModProbe(HardwareInterface):

    def __init__(self, base_dir='/sys/bus/w1/devices/', device_file_name='/w1_slave', serial_no=None, channels=None,
                 *args, **kwargs):
        self.base_dir = base_dir
        self.device_folder = glob.glob(base_dir + '28*')[0]
        self.device_file = self.device_folder + device_file_name
        self.type = self.__name__
        super().__init__(serial_no, channels, *args, **kwargs)

    def create_channel(self):
        return {"mod": ModProbeTemp(self.device_file, self.serial_no)}

    # def read_temp_raw(self):
    #     f = open(self.device_file, 'r')
    #     lines = f.readlines()
    #     f.close()
    #     return lines
    #
    # def read_temp_c(self):
    #     lines = self.read_temp_raw()
    #     while lines[0].strip()[-3:] != 'YES':
    #         time.sleep(0.2)
    #         lines = self.read_temp_raw()
    #     equals_pos = lines[1].find('t=')
    #     if equals_pos != -1:
    #         temp_string = lines[1][equals_pos + 2:]
    #         return float(temp_string) / 1000.0
    #
    # def read_temp_f(self):
    #     temp_c = self.read_temp_c()
    #     return temp_c * 9.0 / 5.0 + 32.0



import asyncio

from OpenHub.homekit_accessories.homkit_sensor_interface import HomeKitSensorInterface
import logging
from OpenHub.globals import driver
from pyhap.const import CATEGORY_CAMERA
from pyhap.camera import Camera as PyHapCamera
from pyhap.camera import VIDEO_CODEC_PARAM_PROFILE_ID_TYPES, VIDEO_CODEC_PARAM_LEVEL_TYPES

logger = logging.getLogger(__name__)


class Camera(PyHapCamera):
    run_debug_message = "Camera State: "

    category = CATEGORY_CAMERA

    def __init__(self, serial_no=None, display_name=None, *args, **kwargs):
        self.category = CATEGORY_CAMERA

        options = {
            "video": {
                "codec": {
                    "profiles": [
                        VIDEO_CODEC_PARAM_PROFILE_ID_TYPES["BASELINE"],
                        VIDEO_CODEC_PARAM_PROFILE_ID_TYPES["MAIN"],
                        VIDEO_CODEC_PARAM_PROFILE_ID_TYPES["HIGH"]
                    ],
                    "levels": [
                        VIDEO_CODEC_PARAM_LEVEL_TYPES['TYPE3_1'],
                        VIDEO_CODEC_PARAM_LEVEL_TYPES['TYPE3_2'],
                        VIDEO_CODEC_PARAM_LEVEL_TYPES['TYPE4_0'],
                    ],
                },
                "resolutions": [
                    # Width, Height, framerate
                    [320, 240, 15],  # Required for Apple Watch
                    [1024, 768, 30],
                    [640, 480, 30],
                    [640, 360, 30],
                    [480, 360, 30],
                    [480, 270, 30],
                    [320, 240, 30],
                    [320, 180, 30],
                ],
            },
            "audio": {
                "codecs": [
                    {
                        'type': 'OPUS',
                        'samplerate': 24,
                    },
                    {
                        'type': 'AAC-eld',
                        'samplerate': 16
                    }
                ],
            },
            "srtp": False,

            # hard code the address if auto-detection does not work as desired: e.g. "192.168.1.226"
            "address": "192.168.3.101",
            'start_stream_cmd': ('gst-launch-1.0 v4l2src ! video/x-h264, '
                                 'width=640, '
                                 'height=480, '
                                 'framerate=30/1 '
                                 '! h264parse ! '
                                 'rtph264pay config-interval=1 pt=96 ! '
                                 'udpsink sync=false host=192.168.3.101 port=8004')
        }


        self.display_name = self.set_display_name(display_name)
        self.serial_no = serial_no

        super().__init__(options=options, driver=driver, display_name=self.display_name, *args, **kwargs)
        # super(HomeKitSensorInterface, self).__init__(serial_no=serial_no, display_name=display_name, *args,
        #                                              **kwargs)

    def set_display_name(self, display_name):
        if display_name is None:
            return "Camera"
        else:
            return display_name

    # def add_functional_service(self):
    #     return self.add_preload_service('C')
    #
    # def add_functional_service_characteristic(self):
    #     # return self.service.configure_char(
    #     #     'On', setter_callback=self.set_relay)
    #     pass
    #     # ### For client extensions ###

    # def get_snapshot(self, image_size):  # pylint: disable=unused-argument, no-self-use
    #     """Return a jpeg of a snapshot from the camera.
    #     Overwrite to implement getting snapshots from your camera.
    #     :param image_size: ``dict`` describing the requested image size. Contains the
    #         keys "image-width" and "image-height"
    #     """
    #     with open(os.path.join(RESOURCE_DIR, 'snapshot.jpg'), 'rb') as fp:
    #         return fp.read()



import asyncio

from OpenHub.homekit_accessories.homkit_sensor_interface import HomeKitSensorInterface
import logging

from pyhap.const import CATEGORY_SWITCH

logger = logging.getLogger(__name__)


class Relay(HomeKitSensorInterface):
    run_debug_message = "Relay State: "

    def __init__(self, serial_no=None, display_name=None, channel_interface_serial_no=None, *args, **kwargs):
        self.category = CATEGORY_SWITCH

        super().__init__(serial_no=serial_no, display_name=display_name,
                         channel_interface_serial_no=channel_interface_serial_no, *args, **kwargs)

    def set_display_name(self, display_name):
        if display_name is None:
            return "Relay"
        else:
            return display_name

    def add_functional_service(self):
        return self.add_preload_service('Relay')

    def add_functional_service_characteristic(self):
        return self.service.configure_char(
            'On', setter_callback=self.set_relay)

    def __setstate__(self, state):
        self.__dict__.update(state)
        # self._gpio_setup(self.pin)

    def set_relay(self, value):
        loop = asyncio.get_running_loop()
        if value:
            loop.create_task(self.channel.turn_on())
        else:
            loop.create_task(self.channel.turn_off())

    async def stop(self):
        await self.channel.turn_off()
        await super().stop()

    async def run(self):
        pass



import logging
from OpenHub.homekit_accessories.homkit_sensor_interface import HomeKitSensorInterface
from OpenHub.calibrators.soil_moisture_voltage_converter import SoilMoistureVoltageConverter


class SoilMoistureSensor(HomeKitSensorInterface):
    logger = logging.getLogger(__name__)
    run_debug_message = "Current Soil Moisture: "

    def __init__(self, serial_no=None, display_name=None, channel_interface_serial_no=None, *args, **kwargs):
        self.scale = float((1 / 65536) * 100)
        super().__init__(serial_no=serial_no, display_name=display_name,
                         channel_interface_serial_no=channel_interface_serial_no, *args, **kwargs)

    # def __init__(self, calibrator=SoilMoistureVoltageConverter(), serial_no=None, display_name=None,
    #              channel_interface_serial_no=None, *args, **kwargs):
    #     self.add_name()
    #     super().__init__(raw_value_converter=calibrator, serial_no=serial_no, display_name=display_name,
    #                      channel_interface_serial_no=channel_interface_serial_no, *args, **kwargs)


    def set_display_name(self, display_name):
        if display_name is None:
            return "Soil Moisture Sensor"
        else:
            return display_name

    def add_name(self):
        if self.display_name is None:
            self.display_name = input(
                "Please Add Display Name for Sensor on Channel: " + str(self.index)) or "SoilMoistureDisplayName" + str(
                self.index)

    def add_functional_service(self):
        return self.add_preload_service("HumiditySensor")

    def add_functional_service_characteristic(self):
        return self.service.get_characteristic('CurrentRelativeHumidity')
    #
    # async def run(self):
    #     voltage = mcpi.get_voltage_for_channel(int(self.index))
    #     self.logger.debug("Voltage: " + str(voltage))
    #     self.set_min_max_voltage(voltage)
    #     output = self.convert_voltage_to_output()
    #     self.logger.debug("Output / Moisture Level: " + str(output))
    #     self.soil_moisture.set_value(output)
    #     # await asyncio.sleep(1)



import os
from subprocess import call
from pathlib import Path

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent
os.chmod(str(BASE_DIR) + '/install.sh', 755)
rc = call(str(BASE_DIR) + "/install.sh")



import logging

from OpenHub import PiPicoGPO
import RPi.GPIO as GPIO


class PiGPO(PiPicoGPO):
    logger = logging.getLogger(__name__)

    def __init__(self, *args, **kwargs):
        super(PiGPO, self).__init__(*args, **kwargs)
        GPIO.setup(self.pin, GPIO.OUT)
        if self.default == 'on':
            GPIO.output(self.pin, GPIO.HIGH)
        elif self.default == 'off':
            GPIO.output(self.pin, GPIO.LOW)



from abc import ABC, abstractmethod
from pyhap.accessory import Accessory
from pyhap.accessory import CATEGORY_OTHER
import logging
from OpenHub.globals import id_channels_map \
    , driver
import uuid


class HomeKitSensorInterface(ABC, Accessory):
    logger = logging.getLogger(__name__)
    category = CATEGORY_OTHER
    scale = None
    index = None
    service = None
    char = None

    serial_no = None
    display_name = None

    channel = None

    raw_value_converter = None

    run_debug_message = "Run Debug Message Not Implemented"

    calibrator = None
    aid = None

    def __init__(self, serial_no=uuid.uuid4(), display_name=None, channel_interface_serial_no=None,
                 raw_value_converter=None, *args, **kwargs):
        ABC.__init__(self)
        self.display_name = self.set_display_name(display_name)
        self.serial_no = serial_no
        self.channel_serial_no = channel_interface_serial_no
        self.channel = id_channels_map[str(self.channel_serial_no)]
        Accessory.__init__(self, driver=driver, display_name=self.display_name,
                           *args, **kwargs)
        # super(Bridge, self).__init__(driver=driver, display_name=display_name,
        #                              *args, **kwargs)
        self.service = self.add_functional_service()
        self.char = self.add_functional_service_characteristic()

        # accessories[str(self.serial_no)] = self
        if raw_value_converter is not None:
            self.raw_value_converter = raw_value_converter
            self.raw_value_converter.add_accessory_serial_no(self.serial_no)

    def add_info_service(self):
        serv_info = self.driver.loader.get_service("AccessoryInformation")
        serv_info.configure_char("Name", value=self.display_name)
        serv_info.configure_char("SerialNumber", value=self.serial_no)
        serv_info.configure_char("Manufacturer", value="BellyFrito")
        serv_info.configure_char("Model", value="SoilTemperatureSensor")
        self.add_service(serv_info)

    @abstractmethod
    def set_display_name(self, display_name):
        pass

    @abstractmethod
    def add_functional_service_characteristic(self):
        pass

    @abstractmethod
    def add_functional_service(self):
        pass

    async def run(self):
        data = await self.channel.get_raw_data()
        self.logger.info(self.display_name + " Output: " + str(data))
        if 'averaged' in data.keys():
            if self.scale is None:
                self.char.set_value(float(data['averaged']))
            else:
                self.char.set_value(self.scale*float(data['averaged']))
